# React Reducer Context

[![license](https://img.shields.io/github/license/mashape/apistatus.svg)](/LICENSE.txt) ![GitHub package.json version](https://img.shields.io/github/package-json/v/gmullerb/react-reducer-context.svg?logo=npm) ![coverage](https://gitlab.com/gmullerb/react-reducer-context/badges/master/coverage.svg)

**React Component to manage State through reducers using contexts and hooks.**

This project is licensed under the terms of the [MIT license](/LICENSE.txt).
__________________

## Quick Start

1 . Apply the plugin:

`package.json`:

```json
  ..
  "dependencies": {
    "react-reducer-context": "1.0.0",
    ..
```

2 . Create and Use a `ReducerContext` component to manage state:

**`SomeReducerContext.jsx`**:

```jsx
import React, { createContext } from 'react'
import ReducerContext from 'react-reducer-context'

const initialState = 0

function reduce(prevState, action) {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

const someReducerContext = createContext(null)

function SomeReducerContext({ children }) {
  return (
    <ReducerContext
      context={someReducerContext}
      reducer={reduce}
      initialState={initialState}
    >
      {children}
    </ReducerContext>
  )
}

export {
  someReducerContext as default,
  SomeReducerContext
}

```

`SomeContainer.jsx`:

```jsx
import SomeComponent1 from './path/to/SomeComponent1'
import SomeComponent2 from './path/to/SomeComponent2'
import SomeComponentN from './path/to/SomeComponentN'
import { SomeReducerContext } from '../path/to/SomeReducerContext'
import React from 'react'

export default function SomeContainer() {
  return (
    <SomeReducerContext>
      <SomeComponent1/>
      <SomeComponent2/>
      <SomeComponentN/>
    </SomeReducerContext>
  )
}
```

`SomeComponent1.jsx` [1]:

```jsx
import someReducerContext from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'

export default function SomeComponent1() {
  const [, dispatch] = useContext(someReducerContext)
  return (
    <button onClick={() => dispatch('ACTION1')}>
      Go up!
    </button>
  )
}
```

`SomeComponent2.jsx` [1]:

```jsx
import someReducerContext from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'

export default function SomeComponent2() {
  const [, dispatch] = useContext(someReducerContext)
  return (
    <button onClick={() => dispatch('ACTION2')}>
      Go down!
    </button>
  )
}
```

`SomeComponentN.jsx` [1]:

```jsx
import someReducerContext from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'

export default function SomeComponentN() {
  const [currentState,] = useContext(someReducerContext)
  return (
    <div>
      Current:{currentState}
    </div>
  )
}
```

> [1] Injection can be used in order to improve design, but in favor of quick example this was surrender, look at [Injection](#With-Injection) for injection example.

3 . Jump to [Reference](#Reference), for customization or digging on How it works.
__________________

## Goal

With the introduction of React Hooks, in some way using Flux library was deprecated, react-reducer-context looks to **give a quick and easy alternative using hooks to implement Flux with reducers**.
__________________

## Reference

[`ReducerContext`](src/main/js/ReducerContext.js) is based on [Flux](http://facebook.github.io/flux), an application architecture that handles application states in a unidirectional way.

* Flux is composed basically with:
  * Stores: keeps states of the app (or components).
    * Reducer: function that changes the State based on an Action and the previous State.
  * Actions: triggers changes in Store.
  * Dispatcher: sends Actions to the Store.
    * Mainly the bridge between the Store and Components.

![Flux architecture](readme/flux.svg "Flux architecture")

[`ReducerContext`](src/main/js/ReducerContext.js) is a React "Special" Element that requires 3 properties:

* `context`: constitutes the [React Context](https://reactjs.org/docs/context.html) which will be handle by this component.
  * use `React.createContext(null)` to create the context.
* `reducer`: a function that will receive the current state and an action to produce a new state.
  * internally use [`useReducer` hook](https://reactjs.org/docs/hooks-reference.html#usereducer), which return the current state and a [dispatcher](http://facebook.github.io/flux/docs/dispatcher).
* `initialState`: inception state for the component.

```jsx
  <ReducerContext
    context={someReducerContext}
    reducer={reduce}
    initialState={initialState}
  >
    {children}
  </ReducerContext>
```

Each `ReducerContext` is equivalent to an Flux stream:

![ReducerContext](readme/react-reducer-context.svg "ReducerContext")

`children` elements will be able to access the State and Dispatcher. There are different ways of doing this:

A . Using [`useContext`](https://reactjs.org/docs/hooks-reference.html#usecontext):

```jsx
  const [state, dispatch] = useContext(someReducerContext)
```

e.g.:

```jsx
import someReducerContext from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'

export default function SomeComponent() {
  const [state, dispatch] = useContext(someReducerContext)
  return (
    <button onClick={() => dispatch({
        type: 'SOME_ACTION',
        data: someValue
      })}
    >
      Do something! ({state.someValue})
    </button>
  )
}
```

B . Using [`Context.Consumer`](https://reactjs.org/docs/context.html#contextconsumer):

```jsx
  <someReducerContext.Consumer>
    {
      ([state, dispatch]) => (
        ..
      )
    }
  </someReducerContext.Consumer>
```

e.g.:

```jsx
import someReducerContext from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'

export default function SomeComponent() {
  return (
    <someReducerContext.Consumer>
    {
      ([state, dispatch]) => (
        <button onClick={() => dispatch({
            type: 'SOME_ACTION',
            data: someValue
          })}
        >
          Do something! ({state.someValue})
        </button>
      )
    }
    </someReducerContext.Consumer>
  )
}
```

> There is another way using [`contextType`](https://reactjs.org/docs/context.html#classcontexttype), but is not functional approach, so it is not exposed.  

### Nesting

Based on [React Context](https://reactjs.org/docs/context.html), `ReducerContext` can be nested in layers, in order to have several nested Reducer/State.

```jsx
  <ReducerContext
    context={someReducerContext1}
    reducer={reduce1}
    initialState={initialState1}
  >
    {someChildren}
    <ReducerContext
      context={someReducerContextN}
      reducer={reduceN}
      initialState={initialStateN}
    >
      {moreChildren}
    </ReducerContext>
  </ReducerContext>
```

`moreChildren` can access the State and the Dispatcher of the ReducerContext1 plus the State and the Dispatcher of the ReducerContextN.

![Nested ReducerContext](readme/nexted-reducer-context.svg "Nested ReducerContext")

### Typings

**`react-reducer-context` defines typings for Flow and Typescript**:

* Any can be used without an "special" [1] configuration.

Typings definitions are located together with source files:

* Flow: [`ReducerContext.js.flow`](src/main/js/ReducerContext.js.flow).
* Typescript: [`ReducerContext.d.ts`](src/main/js/ReducerContext.d.ts).

Both provide the following types:

* `ReducerContextValue<S, A>`: defines the type of the value contained in the `React.Context`.
* `ReducerContextDefaultValue<S, A>`: specifies the type of the `React.Context` when created.
  * Essentially is a `ReducerContextValue<S, A>` which also allows a `null` value, which is required when creating the context.
* `ReducerContextProps<S, A>`: defines the properties receive the `ReducerContext`.
* `ReducerContext<S, A>`: specifies the Function React Component structure.

`S`: State type.  
`A`: Action type.  

E.G.:

`SomeReducerContext.jsx` or `SomeReducerContext.tsx`:

```jsx
..

const initialState: number = 0

function reduce(prevState: number, action: string): number {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

const someReducerContext: Context<ReducerContextDefaultValue<number, string>> = createContext(null)

..

```

`SomeComponent.jsx` or `SomeComponent.tsx`:

```jsx
  ..
  const [state, dispatch]: ReducerContextValue<number, string> = useContext(someReducerContext)
  ..
```

* A more "complete" example with Flow can be seen at: [`typingTest.jsx`](src/test/typings/flow/typingTest.jsx).
* A more "complete" example with Typescript can be seen at: [`typingTest.tsx`](src/test/typings/ts/typingTest.tsx).

> [1] Only the usual Flow or Typescript configuration.  
__________________

## Configuration/Using

### Prerequisites

* [React Hooks](https://reactjs.org/docs/hooks-overview.html) => [`"react": "^16.8.0"`](https://www.npmjs.com/package/react).

### Configuration

1 . Add **react-reducer-context** (and prerequisite) to `package.json`:

```json
  ..
  "dependencies": {
    "react": "^16.8.0"
    "react-reducer-context": "1.0.0",
    ..
```

### Using

#### With Injection

2 . Create the `ReducerContext`:

**`SomeReducerContext.jsx`**:

```jsx
import React, { createContext } from 'react'
import ReducerContext from 'react-reducer-context'

const initialState = 0

function reduce(prevState, action) {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

const someReducerContext = createContext(null)

function SomeReducerContext({ children }) {
  return (
    <ReducerContext
      context={someReducerContext}
      reducer={reduce}
      initialState={initialState}
    >
      {children}
    </ReducerContext>
  )
}

export {
  someReducerContext as default,
  SomeReducerContext
}
```

3 . Define some Components:

`SomeComponent1.jsx`:

```jsx
import React from 'react'

export default function SomeComponent1({dispatch}) {
  return (
    <button onClick={() => dispatch('ACTION1')}>
      Go up!
    </button>
  )
}
```

`SomeComponent2.jsx`:

```jsx
import React from 'react'

export default function SomeComponent2({dispatch}) {
  return (
    <button onClick={() => dispatch('ACTION2')}>
      Go down!
    </button>
  )
}
```

`SomeComponentN.jsx`:

```jsx
import React from 'react'

export default function SomeComponentN({currentState}) {
  return (
    <div>
      Current:{currentState}
    </div>
  )
}
```

4 . Create the Container to bridge the `ReducerContext` and the Components:

`SomeContainer.jsx`:

```jsx
import SomeComponent1 from './path/to/SomeComponent1'
import SomeComponent2 from './path/to/SomeComponent2'
import SomeComponentN from './path/to/SomeComponentN'
import someReducerContext, { SomeReducerContext } from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'

export default function SomeContainer() {
  const [state, dispatch] = useContext(someReducerContext)
  return (
    <SomeReducerContext>
      <SomeComponent1 dispatch={dispatch}/>
      <SomeComponent2 dispatch={dispatch}/>
      <SomeComponentN currentState={state}/>
    </SomeReducerContext>
  )
}
```

##### with Flow typings

2 . Create the `ReducerContext`:

**`SomeReducerContext.jsx`**:

```jsx
import React, { createContext } from 'react'
import ReducerContext from 'react-reducer-context'

import type {
  Context,
  Element
} from 'react'
import type { ReducerContextDefaultValue } from 'react-reducer-context'

const initialState: number = 0

function reduce(prevState: number, action: string): number {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

const someReducerContext: Context<ReducerContextDefaultValue<number, string>> = createContext(null)

function SomeReducerContext({ children }: {children: Element<any>}) {
  return (
    <ReducerContext
      context={someReducerContext}
      reducer={reduce}
      initialState={initialState}
    >
      {children}
    </ReducerContext>
  )
}

export {
  someReducerContext as default,
  SomeReducerContext
}
```

3 . Define some Components:

`SomeComponent1.jsx`:

```jsx
import React from 'react'

import type { Dispatcher } from 'react-reducer-context'

export default function SomeComponent1({dispatch}: {dispatch: Dispatcher<string>}) {
  return (
    <button onClick={() => dispatch('ACTION1')}>
      Go up!
    </button>
  )
}
```

`SomeComponent2.jsx`:

```jsx
import React from 'react'

import type { Dispatcher } from 'react-reducer-context'

export default function SomeComponent2({dispatch}: {dispatch: Dispatcher<string>}) {
  return (
    <button onClick={() => dispatch('ACTION2')}>
      Go down!
    </button>
  )
}
```

`SomeComponentN.jsx`:

```jsx
import React from 'react'

export default function SomeComponentN({currentState}: {currentState: number}) {
  return (
    <div>
      Current:{currentState}
    </div>
  )
}
```

4 . Create the Container to bridge the `ReducerContext` and the Components:

`SomeContainer.jsx`:

```jsx
import SomeComponent1 from './path/to/SomeComponent1'
import SomeComponent2 from './path/to/SomeComponent2'
import SomeComponentN from './path/to/SomeComponentN'
import someReducerContext, { SomeReducerContext } from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'

import type { ReducerContextValue } from 'react-reducer-context'

export default function SomeContainer() {
  const [state, dispatch]: ReducerContextValue<number, string> = useContext(someReducerContext)
  return (
    <SomeReducerContext>
      <SomeComponent1 dispatch={dispatch}/>
      <SomeComponent2 dispatch={dispatch}/>
      <SomeComponentN currentState={state}/>
    </SomeReducerContext>
  )
}
```

##### with Typescript typings

2 . Create the `ReducerContext`:

**`SomeReducerContext.tsx`**:

```tsx
import React, {
  Context,
  ReactElement,
  ReactNode
  createContext } from 'react'
import ReducerContext, { ReducerContextDefaultValue } from 'react-reducer-context'

const initialState: number = 0

function reduce(prevState: number, action: string): number {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

const someReducerContext: Context<ReducerContextDefaultValue<number, string>> = createContext(null)

function SomeReducerContext({ children }: {children: ReactNode}): ReactElement {
  return (
    <ReducerContext
      context={someReducerContext}
      reducer={reduce}
      initialState={initialState}
    >
      {children}
    </ReducerContext>
  )
}

export {
  someReducerContext as default,
  SomeReducerContext
}
```

3 . Define some Components:

`SomeComponent1.tsx`:

```tsx
import React from 'react'
import { Dispatcher } from 'react-reducer-context'

export default function SomeComponent1({dispatch}: {dispatch: Dispatcher<string>}) {
  return (
    <button onClick={() => dispatch('ACTION1')}>
      Go up!
    </button>
  )
}
```

`SomeComponent2.tsx`:

```tsx
import React from 'react'
import { Dispatcher } from 'react-reducer-context'

export default function SomeComponent2({dispatch}: {dispatch: Dispatcher<string>}) {
  return (
    <button onClick={() => dispatch('ACTION2')}>
      Go down!
    </button>
  )
}
```

`SomeComponentN.tsx`:

```tsx
import React from 'react'

export default function SomeComponentN({currentState}: {currentState: number}) {
  return (
    <div>
      Current:{currentState}
    </div>
  )
}
```

4 . Create the Container to bridge the `ReducerContext` and the Components:

`SomeContainer.tsx`:

```tsx
import SomeComponent1 from './path/to/SomeComponent1'
import SomeComponent2 from './path/to/SomeComponent2'
import SomeComponentN from './path/to/SomeComponentN'
import someReducerContext, { SomeReducerContext } from '../path/to/SomeReducerContext'
import React, { useContext } from 'react'
import { ReducerContextValue } from 'react-reducer-context'

export default function SomeContainer() {
  const [state, dispatch]: ReducerContextValue<number, string> = useContext(someReducerContext)
  return (
    <SomeReducerContext>
      <SomeComponent1 dispatch={dispatch}/>
      <SomeComponent2 dispatch={dispatch}/>
      <SomeComponentN currentState={state}/>
    </SomeReducerContext>
  )
}
```

#### Extraction of Actions ("Actions Creators") and more

2 . Define the States, the Actions and the Reducer:

`SomeReducer.js`:

```js
const initialState = 0

function reduce(prevState, action) {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

export {
  initialState,
  reduce
}
```

3 . Create the `ReducerContext`:

**`SomeReducerContext.jsx`**:

```jsx
import { initialState, reduce } from '../path/to/SomeReducer'
import React, { createContext } from 'react'
import ReducerContext from 'react-reducer-context'

const someReducerContext = createContext(null)

function SomeReducerContext({ children }) {
  return (
    <ReducerContext
      context={someReducerContext}
      reducer={reduce}
      initialState={initialState}
    >
      {children}
    </ReducerContext>
  )
}

export {
  someReducerContext as default,
  SomeReducerContext
}
```

3 . Define the Actions Creators through a custom React Hook:

`SomeActions.js`:

```js
import someReducerContext from '../path/to/SomeReducerContext'
import { useContext } from 'react'

export default function useActions() {
  const [state, dispatch] = useContext(someReducerContext)
  return {
    state,
    actions: {
      goUp: () => {
        dispatch('ACTION1')
      },
      goDown: () => {
        dispatch('ACTION2')
      }
    }
  }
}
```

4 . Define some Components:

`SomeComponent1.jsx`:

```jsx
import React from 'react'

export default function SomeComponent1({onClick}) {
  return (
    <button onClick={onClick}>
      Go up!
    </button>
  )
}
```

`SomeComponent2.jsx`:

```jsx
import React from 'react'

export default function SomeComponent2({onClick}) {
  return (
    <button onClick={onClick}>
      Go down!
    </button>
  )
}
```

`SomeComponentN.jsx`:

```jsx
import React from 'react'

export default function SomeComponentN({currentState}) {
  return (
    <div>
      Current:{currentState}
    </div>
  )
}
```

5 . Create the Container to bridge the `ReducerContext` and the Components:

`SomeContainer.jsx`:

```jsx
import SomeComponent1 from './path/to/SomeComponent1'
import SomeComponent2 from './path/to/SomeComponent2'
import SomeComponentN from './path/to/SomeComponentN'
import useActions from '../path/to/SomeActions'
import React, { useContext } from 'react'

export default function SomeContainer() {
  const actions = useActions(someReducerContext)
  return (
    <SomeReducerContext>
      <SomeComponent1 onClick={actions.goUp}/>
      <SomeComponent2 onClick={actions.goDown}/>
      <SomeComponentN currentState={actions.state}/>
    </SomeReducerContext>
  )
}
```

##### with Flow typings

2 . Define the States, the Actions and the Reducer:

`SomeReducer.js`:

```js
const initialState: number = 0

function reduce(prevState: number, action: string): number {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

export {
  initialState,
  reduce
}
```

3 . Create the `ReducerContext`:

**`SomeReducerContext.jsx`**:

```jsx
import { initialState, reduce } from '../path/to/SomeReducer'
import React, { createContext } from 'react'
import ReducerContext from 'react-reducer-context'

import type {
  Context,
  Element
} from 'react'
import type { ReducerContextDefaultValue } from 'react-reducer-context'

const someReducerContext: Context<ReducerContextDefaultValue<number, string>> = createContext(null)

function SomeReducerContext({ children }: {children: Element<any>}) {
  return (
    <ReducerContext
      context={someReducerContext}
      reducer={reduce}
      initialState={initialState}
    >
      {children}
    </ReducerContext>
  )
}

export {
  someReducerContext as default,
  SomeReducerContext
}
```

3 . Define the Actions Creators through a custom React Hook:

`SomeActions.js`:

```js
import someReducerContext from '../path/to/SomeReducerContext'
import { useContext } from 'react'

import type { ReducerContextValue } from 'react-reducer-context'

interface SomeActions {
  goUp: () => void;
  goDown: () => void;
}

interface UseActions {
  state: number;
  actions: SomeActions;
}

export default function useActions(): SomeActions {
  const [state, dispatch]: ReducerContextValue<number, string> = useContext(someReducerContext)
  return {
    state,
    actions: {
      goUp: () => {
        dispatch('ACTION1')
      },
      goDown: () => {
        dispatch('ACTION2')
      }
    }
  }
}

export {
  SomeActions,
  UseActions,
  useActions as default
}
```

4 . Define some Components:

`SomeComponent1.jsx`:

```jsx
import React from 'react'

export default function SomeComponent1({onClick}: {onClick: () => void}) {
  return (
    <button onClick={onClick}>
      Go up!
    </button>
  )
}
```

`SomeComponent2.jsx`:

```jsx
import React from 'react'

export default function SomeComponent2({onClick}: {onClick: () => void}) {
  return (
    <button onClick={onClick}>
      Go down!
    </button>
  )
}
```

`SomeComponentN.jsx`:

```jsx
import React from 'react'

export default function SomeComponentN({currentState}: {currentState: number}) {
  return (
    <div>
      Current:{currentState}
    </div>
  )
}
```

5 . Create the Container to bridge the `ReducerContext` and the Components:

`SomeContainer.jsx`:

```jsx
import SomeComponent1 from './path/to/SomeComponent1'
import SomeComponent2 from './path/to/SomeComponent2'
import SomeComponentN from './path/to/SomeComponentN'
import useActions, { UseActions } from '../path/to/SomeActions'
import React, { useContext } from 'react'

export default function SomeContainer() {
  const actions: UseActions = useActions(someReducerContext)
  return (
    <SomeReducerContext>
      <SomeComponent1 onClick={actions.goUp}/>
      <SomeComponent2 onClick={actions.goDown}/>
      <SomeComponentN currentState={actions.state}/>
    </SomeReducerContext>
  )
}
```

##### with Typescript typings

2 . Define the States, the Actions and the Reducer:

`SomeReducer.ts`:

```ts
const initialState: number = 0

function reduce(prevState: number, action: string): number {
  switch (action) {
    case 'ACTION1':
      return prevState++
    case 'ACTION2':
      return prevState--
    default:
      return prevState
  }
}

export {
  initialState,
  reduce
}
```

3 . Create the `ReducerContext`:

**`SomeReducerContext.tsx`**:

```tsx
import { initialState, reduce } from '../path/to/SomeReducer'
import React, {
  Context,
  ReactElement,
  ReactNode,
  createContext
} from 'react'
import ReducerContext, { ReducerContextDefaultValue } from 'react-reducer-context'

const someReducerContext: Context<ReducerContextDefaultValue<number, string>> = createContext(null)

function SomeReducerContext({ children }: {children: ReactNode}): ReactElement {
  return (
    <ReducerContext
      context={someReducerContext}
      reducer={reduce}
      initialState={initialState}
    >
      {children}
    </ReducerContext>
  )
}

export {
  someReducerContext as default,
  SomeReducerContext
}
```

3 . Define the Actions Creators through a custom React Hook:

`SomeActions.ts`:

```ts
import someReducerContext from '../path/to/SomeReducerContext'
import { useContext } from 'react'
import { ReducerContextValue } from 'react-reducer-context'

interface SomeActions {
  goUp: () => void;
  goDown: () => void;
}

interface UseActions {
  state: number;
  actions: SomeActions;
}

function useActions(): SomeActions {
  const [state, dispatch]: ReducerContextValue<number, string> = useContext(someReducerContext)
  return {
    state,
    actions: {
      goUp: () => {
        dispatch('ACTION1')
      },
      goDown: () => {
        dispatch('ACTION2')
      }
    }
  }
}

export {
  SomeActions,
  UseActions,
  useActions as default
}
```

4 . Define some Components:

`SomeComponent1.tsx`:

```tsx
import React from 'react'

export default function SomeComponent1({onClick}: {onClick: () => void}) {
  return (
    <button onClick={onClick}>
      Go up!
    </button>
  )
}
```

`SomeComponent2.tsx`:

```tsx
import React from 'react'

export default function SomeComponent2({onClick}: {onClick: () => void}) {
  return (
    <button onClick={onClick}>
      Go down!
    </button>
  )
}
```

`SomeComponentN.tsx`:

```tsx
import React from 'react'

export default function SomeComponentN({currentState}: {currentState: number}) {
  return (
    <div>
      Current:{currentState}
    </div>
  )
}
```

5 . Create the Container to bridge the `ReducerContext` and the Components:

`SomeContainer.tsx`:

```tsx
import SomeComponent1 from './path/to/SomeComponent1'
import SomeComponent2 from './path/to/SomeComponent2'
import SomeComponentN from './path/to/SomeComponentN'
import useActions, { UseActions } from '../path/to/SomeActions'
import React, { useContext } from 'react'

export default function SomeContainer() {
  const actions: UseActions = useActions(someReducerContext)
  return (
    <SomeReducerContext>
      <SomeComponent1 onClick={actions.goUp}/>
      <SomeComponent2 onClick={actions.goDown}/>
      <SomeComponentN currentState={actions.state}/>
    </SomeReducerContext>
  )
}
```
__________________

## Extending/Developing

[Developing](readme/developing.md)

## Documentation

* [`CHANGELOG.md`](CHANGELOG.md): add information of notable changes for each version here, chronologically ordered [1].

> [1] [Keep a Changelog](http://keepachangelog.com)

## License

[MIT License](/LICENSE.txt)
__________________

## Remember

* Use code style verification tools => Encourages Best Practices, Efficiency, Readability and Learnability.
* Start testing early => Encourages Reliability and Maintainability.
* Code Review everything => Encourages Functional suitability, Performance Efficiency and Teamwork.

## Additional words

Don't forget:

* **Love what you do**.
* **Learn everyday**.
* **Learn yourself**.
* **Share your knowledge**.
* **Learn from the past, dream on the future, live and enjoy the present to the max!**.

At life:

* Let's act, not complain.
* Be flexible.

At work:

* Let's give solutions, not questions.
* Aim to simplicity not intellectualism.
